module eth_ip_udp(clk20, Ethernet_TDp, Ethernet_TDn, Data_In, ARP, StartSending, readram);
input clk20, ARP, StartSending;
input [7:0] Data_In;
output Ethernet_TDp, Ethernet_TDn, readram;

		`include "parameters.v"
 
//---------------------------------------------------------------------------------------
//  IP checksum, big-endian 
parameter IPchecksum1 = ({IPV, IHL}<<8)+ Serv_Type+
								(Len[15:8]<<8)+Len[7:0]+
								(Frame_Id<<8)+Frame_Cnt+
								({Flags, Frag_Ofs_Hi}<<8)+Frag_Ofs_Lo+
								(Time_To_Live<<8)+Packet_Type+
								(IPsource_1<<8)+IPsource_2+
								(IPsource_3<<8)+IPsource_4+
								(IPdestination_1<<8)+IPdestination_2+
								(IPdestination_3<<8)+(IPdestination_4);

parameter IPchecksum2 =  ((IPchecksum1&32'h0000FFFF)+(IPchecksum1>>16));
parameter IPchecksum3 = ~((IPchecksum2&32'h0000FFFF)+(IPchecksum2>>16));
//---------------------------------------------------------------------------------------
// Memory buffer
reg [6:0] rdaddress; // Memory addr bus : Prosiriti za nadogradnje
reg [7:0] udp_data;  // UDP Data bus
reg [7:0] arp_data;	// ARP Data bus
reg [7:0] pkt_data;	// 

always @(posedge clk20) 
	case(rdaddress)

   	`include "ARP_packet.v"

	endcase

always @(posedge clk20) 
	case(rdaddress)

  		`include "UDP_packet.v"
   	
	endcase
 
always @(posedge clk20) if (~ARP) pkt_data<=udp_data;
else if (ARP) pkt_data<=arp_data;
//-----------------------------------------------------------------------------------------
// main part
reg [3:0] ShiftCount;	//16 times for 8 bits  : Samo kod 10_tx
reg SendingPacket;	  // 1 if sending data
always @(posedge clk20) if(StartSending) SendingPacket<=1;
// srediti rel adresu  : Fiksni end of packet: promijeniti 
else if(ShiftCount==14 && rdaddress==7'h48) SendingPacket<=0;	 // after CRC: fiksna adresa, promijeniti 
always @(posedge clk20) ShiftCount <= SendingPacket ? ShiftCount+1 : 15;
wire readram = (ShiftCount==15);		// Provjeriti timing za Rd_En
always @(posedge clk20) if(ShiftCount==15) rdaddress <= SendingPacket ? rdaddress+1 : 0;
reg [7:0] ShiftData;  // 8 bit shift reg
always @(posedge clk20) if(ShiftCount[0]) ShiftData <= readram ? pkt_data : {1'b0, ShiftData[7:1]};
//-------------------------------------------------------------------------------------------
// generate the CRC32
reg [31:0] CRC;
reg CRCflush; 
always @(posedge clk20) if(CRCflush) CRCflush <= SendingPacket; 
else if(readram) CRCflush <= (rdaddress==7'h44);	  // Fiksna adresa: Promijeniti
reg CRCinit; 
always @(posedge clk20) if(readram) CRCinit <= (rdaddress==7);
wire CRCinput = CRCflush ? 0 : (ShiftData[0] ^ CRC[31]);
always @(posedge clk20) if(ShiftCount[0]) CRC <= CRCinit ? ~0 : ({CRC[30:0],1'b0} ^ ({32{CRCinput}} & 32'h04C11DB7));
//------------------------------------------------------------------------------------------------
// generate the NLP : Izbaciti za 100_tx verziju
reg [17:0] LinkPulseCount; 
always @(posedge clk20) LinkPulseCount <= SendingPacket ? 0 : LinkPulseCount+1;
reg LinkPulse; 
always @(posedge clk20) LinkPulse <= &LinkPulseCount[17:1];
//--------------------------------------------------------------------------------------------------
//  shift-register and manchester encoder	 :Staviti kao poseban modul
reg SendingPacketData; 
always @(posedge clk20) SendingPacketData <= SendingPacket;
reg [2:0] idlecount; 
always @(posedge clk20) if(SendingPacketData) idlecount<=0; 
else if(~&idlecount) idlecount<=idlecount+1;
wire dataout = CRCflush ? ~CRC[31] : ShiftData[0];
reg qo; 
always @(posedge clk20) qo <= SendingPacketData ? ~dataout^ShiftCount[0] : 1;
reg qoe; 
always @(posedge clk20) qoe <= SendingPacketData | LinkPulse | (idlecount<6);
reg Ethernet_TDp; always @(posedge clk20) Ethernet_TDp <= (qoe ? qo : 1'b0);
reg Ethernet_TDn; always @(posedge clk20) Ethernet_TDn <= (qoe ? ~qo : 1'b0);
//naci verziju za diff out	 : Rijeseno, za sada neka ostane

endmodule  
